@@ -18,6 +18,8 @@ cd "$DIR/../.." || exit 1
 
 source "${DIR}/../find_maven.sh"
 
+: ${WITH_COVERAGE:="false"}
+
 MAVEN_OPTIONS='-B -fae --no-transfer-progress'
 
 if ! type unionBugs >/dev/null 2>&1 || ! type convertXmlToText >/dev/null 2>&1; then
@@ -26,6 +28,10 @@ if ! type unionBugs >/dev/null 2>&1 || ! type convertXmlToText >/dev/null 2>&1;
   exit $?
 fi
 
+if [[ "${WITH_COVERAGE}" != "true" ]]; then
+  MAVEN_OPTIONS="${MAVEN_OPTIONS} -Djacoco.skip"
+fi
+
 #shellcheck disable=SC2086
 ${MVN} ${MAVEN_OPTIONS} test-compile spotbugs:spotbugs
 rc=$?
