@@ -41,7 +41,9 @@ import org.apache.ratis.proto.RaftProtos.AppendEntriesReplyProto;
 import org.apache.ratis.proto.RaftProtos.AppendEntriesReplyProto.AppendResult;
 import org.apache.ratis.proto.RaftProtos.AppendEntriesRequestProto;
 import org.apache.ratis.proto.RaftProtos.InstallSnapshotReplyProto;
+import org.apache.ratis.proto.RaftProtos.InstallSnapshotReplyProto.InstallSnapshotReplyBodyCase;
 import org.apache.ratis.proto.RaftProtos.InstallSnapshotRequestProto;
+import org.apache.ratis.proto.RaftProtos.InstallSnapshotRequestProto.InstallSnapshotRequestBodyCase;
 import org.apache.ratis.statemachine.SnapshotInfo;
 import org.apache.ratis.util.*;
 import org.slf4j.Logger;
@@ -52,8 +54,8 @@ import java.io.InterruptedIOException;
 import java.util.*;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ConcurrentHashMap;
+import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
-import java.util.concurrent.atomic.AtomicBoolean;
 import java.util.concurrent.atomic.AtomicLong;
 
 /**
@@ -67,6 +69,8 @@ public class GrpcLogAppender extends LogAppenderBase {
     APPEND_LOG_RESPONSE_HANDLER_ON_ERROR
   }
 
+  public static final int INSTALL_SNAPSHOT_NOTIFICATION_INDEX = 0;
+
   private static final Comparator<Long> CALL_ID_COMPARATOR = (left, right) -> {
     // calculate diff in order to take care the possibility of numerical overflow
     final long diff = left - right;
@@ -152,7 +156,7 @@ public class GrpcLogAppender extends LogAppenderBase {
   private final TimeDuration logMessageBatchDuration;
   private final int maxOutstandingInstallSnapshots;
   private final TimeoutExecutor scheduler = TimeoutExecutor.getInstance();
-
+  @SuppressWarnings({"squid:S3077"}) // Suppress volatile for generic type
   private volatile StreamObservers appendLogRequestObserver;
   private final boolean useSeparateHBChannel;
 
@@ -431,7 +435,7 @@ public class GrpcLogAppender extends LogAppenderBase {
   }
 
   private void timeoutAppendRequest(long cid, boolean heartbeat) {
-    final AppendEntriesRequest pending = pendingRequests.handleTimeout(cid, heartbeat);
+    final AppendEntriesRequest pending = pendingRequests.remove(cid, heartbeat);
     if (pending != null) {
       final int errorCount = replyState.process(Event.TIMEOUT);
       LOG.warn("{}: Timed out {}appendEntries, errorCount={}, request={}",
@@ -563,7 +567,7 @@ public class GrpcLogAppender extends LogAppenderBase {
   private class InstallSnapshotResponseHandler implements StreamObserver<InstallSnapshotReplyProto> {
     private final String name = getFollower().getName() + "-" + JavaUtils.getClassSimpleName(getClass());
     private final Queue<Integer> pending;
-    private final AtomicBoolean done = new AtomicBoolean(false);
+    private final CompletableFuture<Void> done = new CompletableFuture<>();
     private final boolean isNotificationOnly;
 
     InstallSnapshotResponseHandler() {
@@ -577,15 +581,35 @@ public class GrpcLogAppender extends LogAppenderBase {
 
     void addPending(InstallSnapshotRequestProto request) {
       try (AutoCloseableLock writeLock = lock.writeLock(caller, LOG::trace)) {
-        pending.offer(request.getSnapshotChunk().getRequestIndex());
+        final int index;
+        if (isNotificationOnly) {
+          Preconditions.assertSame(InstallSnapshotRequestBodyCase.NOTIFICATION,
+                  request.getInstallSnapshotRequestBodyCase(), "request case");
+          index = INSTALL_SNAPSHOT_NOTIFICATION_INDEX;
+        } else {
+          Preconditions.assertSame(InstallSnapshotRequestBodyCase.SNAPSHOTCHUNK,
+                  request.getInstallSnapshotRequestBodyCase(), "request case");
+          index = request.getSnapshotChunk().getRequestIndex();
+        }
+        if (index == 0) {
+          Preconditions.assertTrue(pending.isEmpty(), "pending queue is non-empty before offer for index 0");
+        }
+        pending.offer(index);
       }
     }
 
     void removePending(InstallSnapshotReplyProto reply) {
       try (AutoCloseableLock writeLock = lock.writeLock(caller, LOG::trace)) {
-        final Integer index = pending.poll();
-        Objects.requireNonNull(index, "index == null");
-        Preconditions.assertTrue(index == reply.getRequestIndex());
+        final int index = Objects.requireNonNull(pending.poll(), "index == null");
+        if (isNotificationOnly) {
+          Preconditions.assertSame(InstallSnapshotReplyBodyCase.SNAPSHOTINDEX,
+                  reply.getInstallSnapshotReplyBodyCase(), "reply case");
+          Preconditions.assertSame(INSTALL_SNAPSHOT_NOTIFICATION_INDEX, (int) index, "poll index");
+        } else {
+          Preconditions.assertSame(InstallSnapshotReplyBodyCase.REQUESTINDEX,
+                  reply.getInstallSnapshotReplyBodyCase(), "reply case");
+          Preconditions.assertSame(reply.getRequestIndex(), (int) index, "poll index");
+        }
       }
     }
 
@@ -604,12 +628,18 @@ public class GrpcLogAppender extends LogAppenderBase {
       getServer().getStateMachine().event().notifySnapshotInstalled(result, snapshotIndex, getFollower().getPeer());
     }
 
-    boolean isDone() {
-      return done.get();
+    void waitForResponse() {
+      try {
+        done.get();
+      } catch (InterruptedException e) {
+        Thread.currentThread().interrupt();
+      } catch (ExecutionException e) {
+        throw new IllegalStateException("Failed to complete " + name, e);
+      }
     }
 
     void close() {
-      done.set(true);
+      done.complete(null);
       notifyLogAppender();
     }
 
@@ -678,6 +708,8 @@ public class GrpcLogAppender extends LogAppenderBase {
           LOG.error("Unrecognized the reply result {}: Leader is {}, follower is {}",
               reply.getResult(), getServer().getId(), getFollowerId());
           break;
+        case SNAPSHOT_EXPIRED:
+          LOG.warn("{}: Follower could not install snapshot as it is expired.", this);
         default:
           break;
       }
@@ -743,14 +775,7 @@ public class GrpcLogAppender extends LogAppenderBase {
       }
       return;
     }
-
-    while (isRunning() && !responseHandler.isDone()) {
-      try {
-        getEventAwaitForSignal().await();
-      } catch (InterruptedException ignored) {
-        Thread.currentThread().interrupt();
-      }
-    }
+    responseHandler.waitForResponse();
 
     if (responseHandler.hasAllResponse()) {
       getFollower().setSnapshotIndex(snapshot.getTermIndex().getIndex());
@@ -788,14 +813,7 @@ public class GrpcLogAppender extends LogAppenderBase {
       }
       return;
     }
-
-    while (isRunning() && !responseHandler.isDone()) {
-      try {
-        getEventAwaitForSignal().await();
-      } catch (InterruptedException ignored) {
-        Thread.currentThread().interrupt();
-      }
-    }
+    responseHandler.waitForResponse();
   }
 
   /**
@@ -839,6 +857,7 @@ public class GrpcLogAppender extends LogAppenderBase {
 
   static class AppendEntriesRequest {
     private final Timekeeper timer;
+    @SuppressWarnings({"squid:S3077"}) // Suppress volatile for generic type
     private volatile Timekeeper.Context timerContext;
 
     private final long callId;
@@ -847,7 +866,7 @@ public class GrpcLogAppender extends LogAppenderBase {
 
     private final TermIndex firstEntry;
     private final TermIndex lastEntry;
-
+    @SuppressWarnings({"squid:S3077"}) // Suppress volatile for generic type
     private volatile Timestamp sendTime;
 
     AppendEntriesRequest(AppendEntriesRequestProto proto, RaftPeerId followerId, GrpcServerMetrics grpcServerMetrics) {
@@ -930,9 +949,5 @@ public class GrpcLogAppender extends LogAppenderBase {
     AppendEntriesRequest remove(long cid, boolean isHeartbeat) {
       return isHeartbeat ? heartbeats.remove(cid): logRequests.remove(cid);
     }
-
-    public AppendEntriesRequest handleTimeout(long callId, boolean heartbeat) {
-      return heartbeat ? heartbeats.remove(callId) : logRequests.get(callId);
-    }
   }
 }
