@@ -21,6 +21,7 @@ package org.apache.ratis.netty.client;
 import org.apache.ratis.client.DataStreamClientRpc;
 import org.apache.ratis.client.RaftClientConfigKeys;
 import org.apache.ratis.conf.RaftProperties;
+import org.apache.ratis.datastream.impl.DataStreamRequestByteBuf;
 import org.apache.ratis.datastream.impl.DataStreamRequestByteBuffer;
 import org.apache.ratis.datastream.impl.DataStreamRequestFilePositionCount;
 import org.apache.ratis.io.StandardWriteOption;
@@ -52,7 +53,6 @@ import org.apache.ratis.thirdparty.io.netty.channel.socket.SocketChannel;
 import org.apache.ratis.thirdparty.io.netty.handler.codec.ByteToMessageDecoder;
 import org.apache.ratis.thirdparty.io.netty.handler.codec.MessageToMessageEncoder;
 import org.apache.ratis.thirdparty.io.netty.handler.ssl.SslContext;
-import org.apache.ratis.thirdparty.io.netty.util.concurrent.ScheduledFuture;
 import org.apache.ratis.util.JavaUtils;
 import org.apache.ratis.util.MemoizedSupplier;
 import org.apache.ratis.util.NetUtils;
@@ -370,6 +370,7 @@ public class NettyClientStreamRpc implements DataStreamClientRpc {
         p.addLast(ENCODER);
         p.addLast(ENCODER_FILE_POSITION_COUNT);
         p.addLast(ENCODER_BYTE_BUFFER);
+        p.addLast(ENCODER_BYTE_BUF);
         p.addLast(newDecoder());
         p.addLast(handler);
       }
@@ -386,6 +387,16 @@ public class NettyClientStreamRpc implements DataStreamClientRpc {
     }
   }
 
+  static final MessageToMessageEncoder<DataStreamRequestByteBuf> ENCODER_BYTE_BUF = new EncoderByteBuf();
+
+  @ChannelHandler.Sharable
+  static class EncoderByteBuf extends MessageToMessageEncoder<DataStreamRequestByteBuf> {
+    @Override
+    protected void encode(ChannelHandlerContext context, DataStreamRequestByteBuf request, List<Object> out) {
+      NettyDataStreamUtils.encodeDataStreamRequestByteBuf(request, out::add, context.alloc());
+    }
+  }
+
   static final MessageToMessageEncoder<DataStreamRequestFilePositionCount> ENCODER_FILE_POSITION_COUNT
       = new EncoderFilePositionCount();
 
@@ -454,15 +465,13 @@ public class NettyClientStreamRpc implements DataStreamClientRpc {
         LOG.debug("{}: write after {}", this, request);
 
         final TimeDuration timeout = isClose ? closeTimeout : requestTimeout;
-        // if reply success cancel this future
-        final ScheduledFuture<?> timeoutFuture = channel.eventLoop().schedule(() -> {
+        replyEntry.scheduleTimeout(() -> channel.eventLoop().schedule(() -> {
           if (!f.isDone()) {
             f.completeExceptionally(new TimeoutIOException(
-                "Timeout " + timeout + ": Failed to send " + request + " channel: " + channel));
+                "Timeout " + timeout + ": Failed to send " + request + " via channel " + channel));
             replyMap.fail(requestEntry);
           }
-        }, timeout.toLong(timeout.getUnit()), timeout.getUnit());
-        replyEntry.setTimeoutFuture(timeoutFuture);
+        }, timeout.getDuration(), timeout.getUnit()));
       }
     });
     return f;
