@@ -18,10 +18,12 @@
 
 package org.apache.ratis.netty.server;
 
-import org.apache.ratis.client.DataStreamOutputRpc;
+import org.apache.ratis.client.RaftClientConfigKeys;
 import org.apache.ratis.client.impl.ClientProtoUtils;
+import org.apache.ratis.client.impl.DataStreamClientImpl.DataStreamOutputImpl;
 import org.apache.ratis.conf.RaftProperties;
 import org.apache.ratis.datastream.impl.DataStreamReplyByteBuffer;
+import org.apache.ratis.datastream.impl.DataStreamRequestByteBuf;
 import org.apache.ratis.io.StandardWriteOption;
 import org.apache.ratis.io.WriteOption;
 import org.apache.ratis.metrics.Timekeeper;
@@ -69,13 +71,10 @@ import java.nio.ByteBuffer;
 import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
-import java.util.Map;
 import java.util.Optional;
 import java.util.Set;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.CompletionException;
-import java.util.concurrent.ConcurrentHashMap;
-import java.util.concurrent.ConcurrentMap;
 import java.util.concurrent.Executor;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.atomic.AtomicReference;
@@ -111,12 +110,12 @@ public class DataStreamManagement {
   }
 
   static class RemoteStream {
-    private final DataStreamOutputRpc out;
+    private final DataStreamOutputImpl out;
     private final AtomicReference<CompletableFuture<DataStreamReply>> sendFuture
         = new AtomicReference<>(CompletableFuture.completedFuture(null));
     private final RequestMetrics metrics;
 
-    RemoteStream(DataStreamOutputRpc out, RequestMetrics metrics) {
+    RemoteStream(DataStreamOutputImpl out, RequestMetrics metrics) {
       this.metrics = metrics;
       this.out = out;
     }
@@ -132,7 +131,7 @@ public class DataStreamManagement {
     CompletableFuture<DataStreamReply> write(DataStreamRequestByteBuf request, Executor executor) {
       final Timekeeper.Context context = metrics.start();
       return composeAsync(sendFuture, executor,
-          n -> out.writeAsync(request.slice().nioBuffer(), addFlush(request.getWriteOptionList()))
+          n -> out.writeAsync(request.slice().retain(), addFlush(request.getWriteOptionList()))
               .whenComplete((l, e) -> metrics.stop(context, e == null)));
     }
   }
@@ -142,20 +141,20 @@ public class DataStreamManagement {
     private final boolean primary;
     private final LocalStream local;
     private final Set<RemoteStream> remotes;
-    private final RaftServer server;
+    private final Division division;
     private final AtomicReference<CompletableFuture<Void>> previous
         = new AtomicReference<>(CompletableFuture.completedFuture(null));
 
-    StreamInfo(RaftClientRequest request, boolean primary, CompletableFuture<DataStream> stream, RaftServer server,
-        CheckedBiFunction<RaftClientRequest, Set<RaftPeer>, Set<DataStreamOutputRpc>, IOException> getStreams,
+    StreamInfo(RaftClientRequest request, boolean primary, CompletableFuture<DataStream> stream, Division division,
+        CheckedBiFunction<RaftClientRequest, Set<RaftPeer>, Set<DataStreamOutputImpl>, IOException> getStreams,
         Function<RequestType, RequestMetrics> metricsConstructor)
         throws IOException {
       this.request = request;
       this.primary = primary;
       this.local = new LocalStream(stream, metricsConstructor.apply(RequestType.LOCAL_WRITE));
-      this.server = server;
-      final Set<RaftPeer> successors = getSuccessors(server.getId());
-      final Set<DataStreamOutputRpc> outs = getStreams.apply(request, successors);
+      this.division = division;
+      final Set<RaftPeer> successors = getSuccessors(division.getId());
+      final Set<DataStreamOutputImpl> outs = getStreams.apply(request, successors);
       this.remotes = outs.stream()
           .map(o -> new RemoteStream(o, metricsConstructor.apply(RequestType.REMOTE_WRITE)))
           .collect(Collectors.toSet());
@@ -169,16 +168,12 @@ public class DataStreamManagement {
       return request;
     }
 
-    Division getDivision() throws IOException {
-      return server.getDivision(request.getRaftGroupId());
+    Division getDivision() {
+      return division;
     }
 
     Collection<CommitInfoProto> getCommitInfos() {
-      try {
-        return getDivision().getCommitInfos();
-      } catch (IOException e) {
-        throw new IllegalStateException(e);
-      }
+      return getDivision().getCommitInfos();
     }
 
     boolean isPrimary() {
@@ -198,7 +193,7 @@ public class DataStreamManagement {
       return JavaUtils.getClassSimpleName(getClass()) + ":" + request;
     }
 
-    private Set<RaftPeer> getSuccessors(RaftPeerId peerId) throws IOException {
+    private Set<RaftPeer> getSuccessors(RaftPeerId peerId) {
       final RaftConfiguration conf = getDivision().getRaftConf();
       final RoutingTable routingTable = request.getRoutingTable();
 
@@ -210,7 +205,7 @@ public class DataStreamManagement {
         // Default start topology
         // get the other peers from the current configuration
         return conf.getCurrentPeers().stream()
-            .filter(p -> !p.getId().equals(server.getId()))
+            .filter(p -> !p.getId().equals(division.getId()))
             .collect(Collectors.toSet());
       }
 
@@ -218,55 +213,14 @@ public class DataStreamManagement {
     }
   }
 
-  static class StreamMap {
-    private final ConcurrentMap<ClientInvocationId, StreamInfo> map = new ConcurrentHashMap<>();
-
-    StreamInfo computeIfAbsent(ClientInvocationId key, Function<ClientInvocationId, StreamInfo> function) {
-      final StreamInfo info = map.computeIfAbsent(key, function);
-      LOG.debug("computeIfAbsent({}) returns {}", key, info);
-      return info;
-    }
-
-    StreamInfo get(ClientInvocationId key) {
-      final StreamInfo info = map.get(key);
-      LOG.debug("get({}) returns {}", key, info);
-      return info;
-    }
-
-    StreamInfo remove(ClientInvocationId key) {
-      final StreamInfo info = map.remove(key);
-      LOG.debug("remove({}) returns {}", key, info);
-      return info;
-    }
-  }
-
-  public static class ChannelMap {
-    private final Map<ChannelId, Map<ClientInvocationId, ClientInvocationId>> map = new ConcurrentHashMap<>();
-
-    public void add(ChannelId channelId,
-                    ClientInvocationId clientInvocationId) {
-      map.computeIfAbsent(channelId, (e) -> new ConcurrentHashMap<>()).put(clientInvocationId, clientInvocationId);
-    }
-
-    public void remove(ChannelId channelId,
-                       ClientInvocationId clientInvocationId) {
-      Optional.ofNullable(map.get(channelId)).ifPresent((ids) -> ids.remove(clientInvocationId));
-    }
-
-    public Set<ClientInvocationId> remove(ChannelId channelId) {
-      return Optional.ofNullable(map.remove(channelId))
-          .map(Map::keySet)
-          .orElse(Collections.emptySet());
-    }
-  }
-
   private final RaftServer server;
   private final String name;
 
-  private final StreamMap streams = new StreamMap();
+  private final StreamMap<StreamInfo> streams = new StreamMap<>();
   private final ChannelMap channels;
   private final ExecutorService requestExecutor;
   private final ExecutorService writeExecutor;
+  private final TimeDuration requestTimeout;
 
   private final NettyServerStreamRpcMetrics nettyServerStreamRpcMetrics;
 
@@ -283,6 +237,7 @@ public class DataStreamManagement {
     this.writeExecutor = ConcurrentUtils.newThreadPoolWithMax(useCachedThreadPool,
           RaftServerConfigKeys.DataStream.asyncWriteThreadPoolSize(properties),
           name + "-write-");
+    this.requestTimeout = RaftClientConfigKeys.DataStream.requestTimeout(server.getProperties());
 
     this.nettyServerStreamRpcMetrics = metrics;
   }
@@ -315,12 +270,13 @@ public class DataStreamManagement {
   }
 
   private StreamInfo newStreamInfo(ByteBuf buf,
-      CheckedBiFunction<RaftClientRequest, Set<RaftPeer>, Set<DataStreamOutputRpc>, IOException> getStreams) {
+      CheckedBiFunction<RaftClientRequest, Set<RaftPeer>, Set<DataStreamOutputImpl>, IOException> getStreams) {
     try {
       final RaftClientRequest request = ClientProtoUtils.toRaftClientRequest(
           RaftClientRequestProto.parseFrom(buf.nioBuffer()));
       final boolean isPrimary = server.getId().equals(request.getServerId());
-      return new StreamInfo(request, isPrimary, computeDataStreamIfAbsent(request), server, getStreams,
+      final Division division = server.getDivision(request.getRaftGroupId());
+      return new StreamInfo(request, isPrimary, computeDataStreamIfAbsent(request), division, getStreams,
           getMetrics()::newRequestMetrics);
     } catch (Throwable e) {
       throw new CompletionException(e);
@@ -386,7 +342,7 @@ public class DataStreamManagement {
         .build();
   }
 
-  static void sendReply(List<CompletableFuture<DataStreamReply>> remoteWrites,
+  private void sendReply(List<CompletableFuture<DataStreamReply>> remoteWrites,
       DataStreamRequestByteBuf request, long bytesWritten, Collection<CommitInfoProto> commitInfos,
       ChannelHandlerContext ctx) {
     final boolean success = checkSuccessRemoteWrite(remoteWrites, bytesWritten, request);
@@ -449,17 +405,29 @@ public class DataStreamManagement {
   }
 
   void read(DataStreamRequestByteBuf request, ChannelHandlerContext ctx,
-      CheckedBiFunction<RaftClientRequest, Set<RaftPeer>, Set<DataStreamOutputRpc>, IOException> getStreams) {
+      CheckedBiFunction<RaftClientRequest, Set<RaftPeer>, Set<DataStreamOutputImpl>, IOException> getStreams) {
     LOG.debug("{}: read {}", this, request);
     try {
       readImpl(request, ctx, getStreams);
     } catch (Throwable t) {
       replyDataStreamException(t, request, ctx);
+      removeDataStream(ClientInvocationId.valueOf(request.getClientId(), request.getStreamId()), null);
+    }
+  }
+
+  private void removeDataStream(ClientInvocationId invocationId, StreamInfo info) {
+    final StreamInfo removed = streams.remove(invocationId);
+    if (info == null) {
+      info = removed;
+    }
+    if (info != null) {
+      info.getDivision().getDataStreamMap().remove(invocationId);
+      info.getLocal().cleanUp();
     }
   }
 
   private void readImpl(DataStreamRequestByteBuf request, ChannelHandlerContext ctx,
-      CheckedBiFunction<RaftClientRequest, Set<RaftPeer>, Set<DataStreamOutputRpc>, IOException> getStreams) {
+      CheckedBiFunction<RaftClientRequest, Set<RaftPeer>, Set<DataStreamOutputImpl>, IOException> getStreams) {
     final boolean close = request.getWriteOptionList().contains(StandardWriteOption.CLOSE);
     ClientInvocationId key =  ClientInvocationId.valueOf(request.getClientId(), request.getStreamId());
 
@@ -473,8 +441,6 @@ public class DataStreamManagement {
           () -> newStreamInfo(request.slice(), getStreams));
       info = streams.computeIfAbsent(key, id -> supplier.get());
       if (!supplier.isInitialized()) {
-        final StreamInfo removed = streams.remove(key);
-        removed.getLocal().cleanUp();
         throw new IllegalStateException("Failed to create a new stream for " + request
             + " since a stream already exists Key: " + key + " StreamInfo:" + info);
       }
@@ -512,9 +478,8 @@ public class DataStreamManagement {
         }, requestExecutor)).whenComplete((v, exception) -> {
       try {
         if (exception != null) {
-          final StreamInfo removed = streams.remove(key);
           replyDataStreamException(server, exception, info.getRequest(), request, ctx);
-          removed.getLocal().cleanUp();
+          removeDataStream(key, info);
         }
       } finally {
         request.release();
@@ -531,10 +496,15 @@ public class DataStreamManagement {
     Preconditions.assertTrue(request.getStreamOffset() == reply.getStreamOffset());
   }
 
-  static boolean checkSuccessRemoteWrite(List<CompletableFuture<DataStreamReply>> replyFutures, long bytesWritten,
+  private boolean checkSuccessRemoteWrite(List<CompletableFuture<DataStreamReply>> replyFutures, long bytesWritten,
       final DataStreamRequestByteBuf request) {
     for (CompletableFuture<DataStreamReply> replyFuture : replyFutures) {
-      final DataStreamReply reply = replyFuture.join();
+      final DataStreamReply reply;
+      try {
+        reply = replyFuture.get(requestTimeout.getDuration(), requestTimeout.getUnit());
+      } catch (Exception e) {
+        throw new CompletionException("Failed to get reply for bytesWritten=" + bytesWritten + ", " + request, e);
+      }
       assertReplyCorrespondingToRequest(request, reply);
       if (!reply.isSuccess()) {
         LOG.warn("reply is not success, request: {}", request);
