@@ -77,9 +77,14 @@ public interface RaftServerConfigKeys {
     return getTimeDuration(properties.getTimeDuration(SLEEP_DEVIATION_THRESHOLD_DEFAULT.getUnit()),
         SLEEP_DEVIATION_THRESHOLD_KEY, SLEEP_DEVIATION_THRESHOLD_DEFAULT, getDefaultLog());
   }
+  /** @deprecated use {@link #setSleepDeviationThreshold(RaftProperties, TimeDuration)}. */
+  @Deprecated
   static void setSleepDeviationThreshold(RaftProperties properties, int thresholdMs) {
     setInt(properties::setInt, SLEEP_DEVIATION_THRESHOLD_KEY, thresholdMs);
   }
+  static void setSleepDeviationThreshold(RaftProperties properties, TimeDuration threshold) {
+    setTimeDuration(properties::setTimeDuration, SLEEP_DEVIATION_THRESHOLD_KEY, threshold);
+  }
 
   String CLOSE_THRESHOLD_KEY = PREFIX + ".close.threshold";
   TimeDuration CLOSE_THRESHOLD_DEFAULT = TimeDuration.valueOf(60, TimeUnit.SECONDS);
@@ -87,8 +92,13 @@ public interface RaftServerConfigKeys {
     return getTimeDuration(properties.getTimeDuration(CLOSE_THRESHOLD_DEFAULT.getUnit()),
         CLOSE_THRESHOLD_KEY, CLOSE_THRESHOLD_DEFAULT, getDefaultLog());
   }
-  static void setCloseThreshold(RaftProperties properties, int thresholdMs) {
-    setInt(properties::setInt, CLOSE_THRESHOLD_KEY, thresholdMs);
+  /** @deprecated use {@link #setCloseThreshold(RaftProperties, TimeDuration)}. */
+  @Deprecated
+  static void setCloseThreshold(RaftProperties properties, int thresholdSec) {
+    setInt(properties::setInt, CLOSE_THRESHOLD_KEY, thresholdSec);
+  }
+  static void setCloseThreshold(RaftProperties properties, TimeDuration threshold) {
+    setTimeDuration(properties::setTimeDuration, CLOSE_THRESHOLD_KEY, threshold);
   }
 
   /**
@@ -465,6 +475,18 @@ public interface RaftServerConfigKeys {
       setBoolean(properties::setBoolean, ASYNC_FLUSH_ENABLED_KEY, asyncFlush);
     }
 
+    /** Log metadata can guarantee that a server can recover commit index and state machine
+     * even if a majority of servers are dead by consuming a little extra space. */
+    String LOG_METADATA_ENABLED_KEY = PREFIX + ".log-metadata.enabled";
+    boolean LOG_METADATA_ENABLED_DEFAULT = true;
+    static boolean logMetadataEnabled(RaftProperties properties) {
+      return getBoolean(properties::getBoolean,
+          LOG_METADATA_ENABLED_KEY, LOG_METADATA_ENABLED_DEFAULT, getDefaultLog());
+    }
+    static void setLogMetadataEnabled(RaftProperties properties, boolean logMetadata) {
+      setBoolean(properties::setBoolean, LOG_METADATA_ENABLED_KEY, logMetadata);
+    }
+
     /** The policy to handle corrupted raft log. */
     enum CorruptionPolicy {
       /** Rethrow the exception. */
@@ -647,6 +669,18 @@ public interface RaftServerConfigKeys {
       setBoolean(properties::setBoolean, TRIGGER_WHEN_STOP_ENABLED_KEY, triggerWhenStopEnabled);
     }
 
+    /** whether trigger snapshot when remove raft server */
+    String TRIGGER_WHEN_REMOVE_ENABLED_KEY = PREFIX + ".trigger-when-remove.enabled";
+    /** by default let the state machine to trigger snapshot when remove */
+    boolean TRIGGER_WHEN_REMOVE_ENABLED_DEFAULT = true;
+    static boolean triggerWhenRemoveEnabled(RaftProperties properties) {
+      return getBoolean(properties::getBoolean,
+          TRIGGER_WHEN_REMOVE_ENABLED_KEY, TRIGGER_WHEN_REMOVE_ENABLED_DEFAULT, getDefaultLog());
+    }
+    static void setTriggerWhenRemoveEnabled(RaftProperties properties, boolean triggerWhenRemoveEnabled) {
+      setBoolean(properties::setBoolean, TRIGGER_WHEN_REMOVE_ENABLED_KEY, triggerWhenRemoveEnabled);
+    }
+
     /** The log index gap between to two snapshot creations. */
     String CREATION_GAP_KEY = PREFIX + ".creation.gap";
     long CREATION_GAP_DEFAULT = 1024;
