@@ -24,6 +24,7 @@ import org.apache.ratis.client.impl.RaftClientTestUtil;
 import org.apache.ratis.conf.RaftProperties;
 import org.apache.ratis.proto.RaftProtos.CommitInfoProto;
 import org.apache.ratis.proto.RaftProtos.LogEntryProto;
+import org.apache.ratis.proto.RaftProtos.ReplicationLevel;
 import org.apache.ratis.protocol.RaftGroupId;
 import org.apache.ratis.protocol.RaftPeer;
 import org.apache.ratis.protocol.exceptions.AlreadyClosedException;
@@ -77,7 +78,7 @@ public abstract class RaftAsyncTests<CLUSTER extends MiniRaftCluster> extends Ba
 
   public static final int NUM_SERVERS = 3;
 
-  private static final DelayLocalExecutionInjection logSyncDelay = RaftServerTestUtil.getLogSyncDelay();
+  private static final DelayLocalExecutionInjection LOG_SYNC_DELAY = RaftServerTestUtil.getLogSyncDelay();
 
   {
     getProperties().setClass(MiniRaftCluster.STATEMACHINE_CLASS_KEY,
@@ -357,6 +358,38 @@ public abstract class RaftAsyncTests<CLUSTER extends MiniRaftCluster> extends Ba
     }
   }
 
+  @Test
+  public void testWriteAsyncCustomReplicationLevel() throws Exception {
+    // verify that send(msg, ALL_COMMITTED) would reply with all servers committed past the log index
+    runWithNewCluster(NUM_SERVERS, this::runTestWriteAsyncCustomReplicationLevel);
+  }
+
+  void runTestWriteAsyncCustomReplicationLevel(CLUSTER cluster) throws Exception {
+    final int numMessages = 20;
+    try (RaftClient client = cluster.createClient()) {
+      RaftTestUtil.waitForLeader(cluster);
+
+      // submit some messages
+      for (int i = 0; i < numMessages; i++) {
+        final String s = "" + i;
+        LOG.info("sendAsync with ALL_COMMITTED " + s);
+        client.async().send(new SimpleMessage(s), ReplicationLevel.ALL_COMMITTED).whenComplete((reply, exception) -> {
+          if (exception != null) {
+            LOG.error("Failed to send message " + s, exception);
+            // reply should be null in case of exception
+            Assert.assertNull(reply);
+            return;
+          }
+          Assert.assertTrue(reply.isSuccess());
+          Assert.assertNull(reply.getException());
+          // verify that all servers have caught up to log index when the reply is returned
+          reply.getCommitInfos().forEach(commitInfoProto ->
+              Assert.assertTrue(commitInfoProto.getCommitIndex() >= reply.getLogIndex()));
+        });
+      }
+    }
+  }
+
   @Test
   public void testRequestTimeout() throws Exception {
     final TimeDuration oldExpiryTime = RaftServerConfigKeys.RetryCache.expiryTime(getProperties());
@@ -372,7 +405,7 @@ public abstract class RaftAsyncTests<CLUSTER extends MiniRaftCluster> extends Ba
   @Test
   public void testStateMachineMetrics() throws Exception {
     runWithNewCluster(NUM_SERVERS, cluster ->
-        RaftBasicTests.testStateMachineMetrics(true, cluster, LOG));
+        RaftBasicTests.runTestStateMachineMetrics(true, cluster));
   }
 
   @Test
@@ -430,7 +463,7 @@ public abstract class RaftAsyncTests<CLUSTER extends MiniRaftCluster> extends Ba
       cluster.getServerAliveStream()
           .filter(impl -> !impl.getInfo().isLeader())
           .map(SimpleStateMachine4Testing::get)
-          .forEach(peer -> logSyncDelay.setDelayMs(peer.getId().toString(), 1000));
+          .forEach(peer -> LOG_SYNC_DELAY.setDelayMs(peer.getId().toString(), 1000));
 
       // trigger append entries request
       client.async().send(new SimpleMessage("abc"));
@@ -446,7 +479,7 @@ public abstract class RaftAsyncTests<CLUSTER extends MiniRaftCluster> extends Ba
 
     } finally {
       // unblock append entries request
-      logSyncDelay.clear();
+      LOG_SYNC_DELAY.clear();
     }
 
     waitForLeader(cluster);
