@@ -17,8 +17,6 @@
  */
 package org.apache.ratis.server.raftlog.segmented;
 
-import static org.junit.Assert.assertTrue;
-
 import org.apache.ratis.BaseTest;
 import org.apache.ratis.RaftTestUtil.SimpleOperation;
 import org.apache.ratis.conf.RaftProperties;
@@ -42,37 +40,49 @@ import org.apache.ratis.server.storage.RaftStorageTestUtils;
 import org.apache.ratis.statemachine.impl.SimpleStateMachine4Testing;
 import org.apache.ratis.statemachine.StateMachine;
 import org.apache.ratis.statemachine.impl.BaseStateMachine;
+import org.apache.ratis.util.CodeInjectionForTesting;
+import org.apache.ratis.util.DataBlockingQueue;
 import org.apache.ratis.util.LifeCycle;
 import org.apache.ratis.util.Slf4jUtils;
 import org.apache.ratis.util.FileUtils;
 import org.apache.ratis.util.JavaUtils;
 import org.apache.ratis.util.SizeInBytes;
 import org.apache.ratis.util.TimeDuration;
-import org.junit.After;
-import org.junit.Assert;
-import org.junit.Before;
-import org.junit.Test;
 
 import java.io.File;
 import java.io.IOException;
 import java.nio.ByteBuffer;
-import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Arrays;
-import java.util.Collection;
 import java.util.Collections;
 import java.util.List;
 import java.util.concurrent.CompletableFuture;
+import java.util.concurrent.ConcurrentLinkedQueue;
+import java.util.concurrent.CountDownLatch;
 import java.util.concurrent.ExecutionException;
 import java.util.concurrent.TimeUnit;
+import java.util.concurrent.TimeoutException;
+import java.util.concurrent.atomic.AtomicInteger;
+import java.util.concurrent.atomic.AtomicReference;
 import java.util.function.LongSupplier;
 import java.util.function.Supplier;
+import java.util.stream.Stream;
 
-import org.junit.runner.RunWith;
-import org.junit.runners.Parameterized;
+import org.junit.jupiter.api.AfterEach;
+import org.junit.jupiter.api.Assertions;
+import org.junit.jupiter.api.BeforeEach;
+import org.junit.jupiter.api.Test;
+import org.junit.jupiter.params.ParameterizedTest;
+import org.junit.jupiter.params.provider.Arguments;
+import org.junit.jupiter.params.provider.MethodSource;
 import org.slf4j.event.Level;
 
-@RunWith(Parameterized.class)
+import static java.lang.Boolean.FALSE;
+import static java.lang.Boolean.TRUE;
+import static org.apache.ratis.server.raftlog.segmented.SegmentedRaftLogWorker.RUN_WORKER;
+import static org.junit.jupiter.api.Assertions.assertTrue;
+import static org.junit.jupiter.params.provider.Arguments.arguments;
+
 public class TestSegmentedRaftLog extends BaseTest {
   static {
     Slf4jUtils.setLogLevel(SegmentedRaftLogWorker.LOG, Level.INFO);
@@ -80,27 +90,21 @@ public class TestSegmentedRaftLog extends BaseTest {
     Slf4jUtils.setLogLevel(SegmentedRaftLog.LOG, Level.INFO);
   }
 
-  private final Boolean smSyncFlush;
-  private final Boolean useAsyncFlush;
-
-  public TestSegmentedRaftLog(Boolean raftLogAsync, Boolean smSync) {
-    this.useAsyncFlush = raftLogAsync;
-    this.smSyncFlush = smSync;
-  }
-
-  @Parameterized.Parameters
-  public static Collection<Boolean[]> data() {
-    return Arrays.asList((new Boolean[][] {{Boolean.FALSE, Boolean.FALSE}, {Boolean.FALSE, Boolean.TRUE},
-        {Boolean.TRUE, Boolean.FALSE}, {Boolean.TRUE, Boolean.TRUE}}));
+  public static Stream<Arguments> data() {
+    return Stream.of(
+        arguments(FALSE, FALSE),
+        arguments(FALSE, TRUE),
+        arguments(TRUE, FALSE),
+        arguments(TRUE, TRUE));
   }
 
   public static long getOpenSegmentSize(RaftLog raftLog) {
     return ((SegmentedRaftLog)raftLog).getRaftLogCache().getOpenSegment().getTotalFileSize();
   }
 
-  private static final RaftPeerId peerId = RaftPeerId.valueOf("s0");
-  private static final RaftGroupId groupId = RaftGroupId.randomId();
-  private static final RaftGroupMemberId memberId = RaftGroupMemberId.valueOf(peerId, groupId);
+  private static final RaftPeerId PEER_ID = RaftPeerId.valueOf("s0");
+  private static final RaftGroupId GROUP_ID = RaftGroupId.randomId();
+  private static final RaftGroupMemberId MEMBER_ID = RaftGroupMemberId.valueOf(PEER_ID, GROUP_ID);
 
   static class SegmentRange {
     final long start;
@@ -137,7 +141,7 @@ public class TestSegmentedRaftLog extends BaseTest {
 
   static SegmentedRaftLog newSegmentedRaftLog(RaftStorage storage, RaftProperties properties) {
     return SegmentedRaftLog.newBuilder()
-        .setMemberId(memberId)
+        .setMemberId(MEMBER_ID)
         .setStorage(storage)
         .setProperties(properties)
         .build();
@@ -146,20 +150,18 @@ public class TestSegmentedRaftLog extends BaseTest {
   private SegmentedRaftLog newSegmentedRaftLogWithSnapshotIndex(RaftStorage storage, RaftProperties properties,
                                                                 LongSupplier getSnapshotIndexFromStateMachine) {
     return SegmentedRaftLog.newBuilder()
-        .setMemberId(memberId)
+        .setMemberId(MEMBER_ID)
         .setStorage(storage)
         .setSnapshotIndexSupplier(getSnapshotIndexFromStateMachine)
         .setProperties(properties)
         .build();
   }
 
-  @Before
+  @BeforeEach
   public void setup() throws Exception {
     storageDir = getTestDir();
     properties = new RaftProperties();
     RaftServerConfigKeys.setStorageDir(properties,  Collections.singletonList(storageDir));
-    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
-    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     storage = RaftStorageTestUtils.newRaftStorage(storageDir);
     this.segmentMaxSize =
         RaftServerConfigKeys.Log.segmentSizeMax(properties).getSize();
@@ -169,7 +171,7 @@ public class TestSegmentedRaftLog extends BaseTest {
         RaftServerConfigKeys.Log.writeBufferSize(properties).getSizeInt();
   }
 
-  @After
+  @AfterEach
   public void tearDown() throws Exception {
     if (storageDir != null) {
       FileUtils.deleteFully(storageDir.getParentFile());
@@ -211,8 +213,11 @@ public class TestSegmentedRaftLog extends BaseTest {
     return raftLog.get(raftLog.getLastEntryTermIndex().getIndex());
   }
 
-  @Test
-  public void testLoadLogSegments() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testLoadLogSegments(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     // first generate log files
     List<SegmentRange> ranges = prepareRanges(0, 5, 100, 0);
     LogEntryProto[] entries = prepareLog(ranges);
@@ -223,7 +228,7 @@ public class TestSegmentedRaftLog extends BaseTest {
       // check if log entries are loaded correctly
       for (LogEntryProto e : entries) {
         LogEntryProto entry = raftLog.get(e.getIndex());
-        Assert.assertEquals(e, entry);
+        Assertions.assertEquals(e, entry);
       }
 
       final LogEntryHeader[] termIndices = raftLog.getEntries(0, 500);
@@ -236,10 +241,10 @@ public class TestSegmentedRaftLog extends BaseTest {
             }
           })
           .toArray(LogEntryProto[]::new);
-      Assert.assertArrayEquals(entries, entriesFromLog);
-      Assert.assertEquals(entries[entries.length - 1], getLastEntry(raftLog));
+      Assertions.assertArrayEquals(entries, entriesFromLog);
+      Assertions.assertEquals(entries[entries.length - 1], getLastEntry(raftLog));
 
-      final RatisMetricRegistry metricRegistryForLogWorker = RaftLogMetricsBase.createRegistry(memberId);
+      final RatisMetricRegistry metricRegistryForLogWorker = RaftLogMetricsBase.createRegistry(MEMBER_ID);
 
       final DefaultTimekeeperImpl load = (DefaultTimekeeperImpl) metricRegistryForLogWorker.timer("segmentLoadLatency");
       assertTrue(load.getTimer().getMeanRate() > 0);
@@ -266,7 +271,8 @@ public class TestSegmentedRaftLog extends BaseTest {
     return eList;
   }
 
-  static LogEntryProto prepareLogEntry(long term, long index, Supplier<String> stringSupplier, boolean hasStataMachineData) {
+  static LogEntryProto prepareLogEntry(long term, long index, Supplier<String> stringSupplier,
+      boolean hasStataMachineData) {
     final SimpleOperation m = stringSupplier == null?
         new SimpleOperation("m" + index, hasStataMachineData):
         new SimpleOperation(stringSupplier.get(), hasStataMachineData);
@@ -276,8 +282,11 @@ public class TestSegmentedRaftLog extends BaseTest {
   /**
    * Append entry one by one and check if log state is correct.
    */
-  @Test
-  public void testAppendEntry() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testAppendEntry(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     List<SegmentRange> ranges = prepareRanges(0, 5, 200, 0);
     List<LogEntryProto> entries = prepareLogEntries(ranges, null);
 
@@ -331,8 +340,11 @@ public class TestSegmentedRaftLog extends BaseTest {
     }
   }
 
-  @Test
-  public void testAppendEntryAfterPurge() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testAppendEntryAfterPurge(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     List<SegmentRange> ranges = prepareRanges(0, 5, 200, 0);
     List<LogEntryProto> entries = prepareLogEntries(ranges, null);
 
@@ -362,8 +374,11 @@ public class TestSegmentedRaftLog extends BaseTest {
   /**
    * Keep appending entries, make sure the rolling is correct.
    */
-  @Test
-  public void testAppendAndRoll() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testAppendAndRoll(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     RaftServerConfigKeys.Log.setPreallocatedSize(properties, SizeInBytes.valueOf("16KB"));
     RaftServerConfigKeys.Log.setSegmentSizeMax(properties, SizeInBytes.valueOf("128KB"));
 
@@ -382,12 +397,91 @@ public class TestSegmentedRaftLog extends BaseTest {
       raftLog.open(RaftLog.INVALID_LOG_INDEX, null);
       // check if the raft log is correct
       checkEntries(raftLog, entries, 0, entries.size());
-      Assert.assertEquals(9, raftLog.getRaftLogCache().getNumOfSegments());
+      Assertions.assertEquals(9, raftLog.getRaftLogCache().getNumOfSegments());
     }
   }
 
-  @Test
-  public void testTruncate() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testPurgeAfterAppendEntry(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
+    RaftServerConfigKeys.Log.setPurgeGap(properties, 1);
+    RaftServerConfigKeys.Log.setForceSyncNum(properties, 128);
+
+    int startTerm = 0;
+    int endTerm = 2;
+    int segmentSize = 10;
+    long endIndexOfClosedSegment = segmentSize * (endTerm - startTerm - 1);
+    long nextStartIndex = segmentSize * (endTerm - startTerm);
+
+    // append entries and roll logSegment for later purge operation
+    List<SegmentRange> ranges0 = prepareRanges(startTerm, endTerm, segmentSize, 0);
+    List<LogEntryProto> entries0 = prepareLogEntries(ranges0, null);
+    try (SegmentedRaftLog raftLog = newSegmentedRaftLog()) {
+      raftLog.open(RaftLog.INVALID_LOG_INDEX, null);
+      entries0.stream().map(raftLog::appendEntry).forEach(CompletableFuture::join);
+    }
+
+    // test the pattern in the task queue of SegmentedRaftLogWorker: (WriteLog, ..., PurgeLog)
+    List<SegmentRange> ranges = prepareRanges(endTerm - 1, endTerm, 1, nextStartIndex);
+    List<LogEntryProto> entries = prepareLogEntries(ranges, null);
+
+    try (SegmentedRaftLog raftLog = newSegmentedRaftLog()) {
+      final CountDownLatch raftLogOpened = new CountDownLatch(1);
+      final CountDownLatch tasksAdded = new CountDownLatch(1);
+
+      // inject test code to make the pattern (WriteLog, PurgeLog)
+      final ConcurrentLinkedQueue<CompletableFuture<Long>> appendFutures = new ConcurrentLinkedQueue<>();
+      final AtomicReference<CompletableFuture<Long>> purgeFuture = new AtomicReference<>();
+      final AtomicInteger tasksCount = new AtomicInteger(0);
+      CodeInjectionForTesting.put(RUN_WORKER, (localId, remoteId, args) -> {
+        // wait for raftLog to be opened
+        try {
+          if(!raftLogOpened.await(FIVE_SECONDS.getDuration(), FIVE_SECONDS.getUnit())) {
+            throw new TimeoutException();
+          }
+        } catch (InterruptedException | TimeoutException e) {
+          LOG.error("an exception occurred", e);
+          throw new RuntimeException(e);
+        }
+
+        // add WriteLog and PurgeLog tasks
+        entries.stream().map(raftLog::appendEntry).forEach(appendFutures::add);
+        purgeFuture.set(raftLog.purge(endIndexOfClosedSegment));
+
+        tasksCount.set(((DataBlockingQueue<?>) args[0]).getNumElements());
+        tasksAdded.countDown();
+        return true;
+      });
+
+      // open raftLog
+      raftLog.open(RaftLog.INVALID_LOG_INDEX, null);
+      raftLogOpened.countDown();
+
+      // wait for all tasks to be added
+      if(!tasksAdded.await(FIVE_SECONDS.getDuration(), FIVE_SECONDS.getUnit())) {
+        throw new TimeoutException();
+      }
+      Assertions.assertEquals(entries.size() + 1, tasksCount.get());
+
+      // check if the purge task is executed
+      final Long purged = purgeFuture.get().get();
+      LOG.info("purgeIndex = {}, purged = {}", endIndexOfClosedSegment, purged);
+      Assertions.assertEquals(endIndexOfClosedSegment, raftLog.getRaftLogCache().getStartIndex());
+
+      // check if the appendEntry futures are done
+      JavaUtils.allOf(appendFutures).get(FIVE_SECONDS.getDuration(), FIVE_SECONDS.getUnit());
+    } finally {
+      CodeInjectionForTesting.put(RUN_WORKER, (localId, remoteId, args) -> false);
+    }
+  }
+
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testTruncate(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     // prepare the log for truncation
     List<SegmentRange> ranges = prepareRanges(0, 5, 200, 0);
     List<LogEntryProto> entries = prepareLogEntries(ranges, null);
@@ -418,10 +512,10 @@ public class TestSegmentedRaftLog extends BaseTest {
       raftLog.open(RaftLog.INVALID_LOG_INDEX, null);
       // check if the raft log is correct
       if (fromIndex > 0) {
-        Assert.assertEquals(entries.get((int) (fromIndex - 1)),
+        Assertions.assertEquals(entries.get((int) (fromIndex - 1)),
             getLastEntry(raftLog));
       } else {
-        Assert.assertNull(raftLog.getLastEntryTermIndex());
+        Assertions.assertNull(raftLog.getLastEntryTermIndex());
       }
       checkEntries(raftLog, entries, 0, (int) fromIndex);
     }
@@ -432,7 +526,7 @@ public class TestSegmentedRaftLog extends BaseTest {
     if (size > 0) {
       for (int i = offset; i < size + offset; i++) {
         LogEntryProto entry = raftLog.get(expected.get(i).getIndex());
-        Assert.assertEquals(expected.get(i), entry);
+        Assertions.assertEquals(expected.get(i), entry);
       }
       final LogEntryHeader[] termIndices = raftLog.getEntries(
           expected.get(offset).getIndex(),
@@ -448,7 +542,7 @@ public class TestSegmentedRaftLog extends BaseTest {
           .toArray(LogEntryProto[]::new);
       LogEntryProto[] expectedArray = expected.subList(offset, offset + size)
           .stream().toArray(LogEntryProto[]::new);
-      Assert.assertArrayEquals(expectedArray, entriesFromLog);
+      Assertions.assertArrayEquals(expectedArray, entriesFromLog);
     }
   }
 
@@ -489,7 +583,7 @@ public class TestSegmentedRaftLog extends BaseTest {
     int segmentSize = 200;
     long endIndexOfClosedSegment = segmentSize * (endTerm - startTerm - 1) - 1;
     long expectedIndex = segmentSize * (endTerm - startTerm - 1);
-    final RatisMetricRegistry metricRegistryForLogWorker = RaftLogMetricsBase.createRegistry(memberId);
+    final RatisMetricRegistry metricRegistryForLogWorker = RaftLogMetricsBase.createRegistry(MEMBER_ID);
     purgeAndVerify(startTerm, endTerm, segmentSize, 1, endIndexOfClosedSegment, expectedIndex);
     final DefaultTimekeeperImpl purge = (DefaultTimekeeperImpl) metricRegistryForLogWorker.timer("purgeLog");
     assertTrue(purge.getTimer().getCount() > 0);
@@ -505,7 +599,8 @@ public class TestSegmentedRaftLog extends BaseTest {
     purgeAndVerify(startTerm, endTerm, segmentSize, 1000, endIndexOfClosedSegment, expectedIndex);
   }
 
-  private void purgeAndVerify(int startTerm, int endTerm, int segmentSize, int purgeGap, long purgeIndex, long expectedIndex) throws Exception {
+  private void purgeAndVerify(int startTerm, int endTerm, int segmentSize, int purgeGap, long purgeIndex,
+      long expectedIndex) throws Exception {
     List<SegmentRange> ranges = prepareRanges(startTerm, endTerm, segmentSize, 0);
     List<LogEntryProto> entries = prepareLogEntries(ranges, null);
 
@@ -517,21 +612,25 @@ public class TestSegmentedRaftLog extends BaseTest {
       final CompletableFuture<Long> f = raftLog.purge(purgeIndex);
       final Long purged = f.get();
       LOG.info("purgeIndex = {}, purged = {}", purgeIndex, purged);
-      Assert.assertEquals(expectedIndex, raftLog.getRaftLogCache().getStartIndex());
+      Assertions.assertEquals(expectedIndex, raftLog.getRaftLogCache().getStartIndex());
     }
   }
 
   /**
    * Test append with inconsistent entries
    */
-  @Test
-  public void testAppendEntriesWithInconsistency() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testAppendEntriesWithInconsistency(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     // prepare the log for truncation
     List<SegmentRange> ranges = prepareRanges(0, 5, 200, 0);
     List<LogEntryProto> entries = prepareLogEntries(ranges, null);
 
     final RetryCache retryCache = RetryCacheTestUtil.createRetryCache();
-    try (SegmentedRaftLog raftLog = RetryCacheTestUtil.newSegmentedRaftLog(memberId, retryCache, storage, properties)) {
+    try (SegmentedRaftLog raftLog =
+             RetryCacheTestUtil.newSegmentedRaftLog(MEMBER_ID, retryCache, storage, properties)) {
       raftLog.open(RaftLog.INVALID_LOG_INDEX, null);
       entries.forEach(entry -> RetryCacheTestUtil.createEntry(retryCache, entry));
       // append entries to the raftlog
@@ -546,7 +645,8 @@ public class TestSegmentedRaftLog extends BaseTest {
     List<LogEntryProto> newEntries = prepareLogEntries(
         Arrays.asList(r1, r2, r3), null);
 
-    try (SegmentedRaftLog raftLog = RetryCacheTestUtil.newSegmentedRaftLog(memberId, retryCache, storage, properties)) {
+    try (SegmentedRaftLog raftLog =
+             RetryCacheTestUtil.newSegmentedRaftLog(MEMBER_ID, retryCache, storage, properties)) {
       raftLog.open(RaftLog.INVALID_LOG_INDEX, null);
       LOG.info("newEntries[0] = {}", newEntries.get(0));
       final int last = newEntries.size() - 1;
@@ -556,35 +656,39 @@ public class TestSegmentedRaftLog extends BaseTest {
       checkFailedEntries(entries, 650, retryCache);
       checkEntries(raftLog, entries, 0, 650);
       checkEntries(raftLog, newEntries, 100, 100);
-      Assert.assertEquals(newEntries.get(newEntries.size() - 1),
+      Assertions.assertEquals(newEntries.get(newEntries.size() - 1),
           getLastEntry(raftLog));
-      Assert.assertEquals(newEntries.get(newEntries.size() - 1).getIndex(),
+      Assertions.assertEquals(newEntries.get(newEntries.size() - 1).getIndex(),
           raftLog.getFlushIndex());
     }
 
     // load the raftlog again and check
-    try (SegmentedRaftLog raftLog = RetryCacheTestUtil.newSegmentedRaftLog(memberId, retryCache, storage, properties)) {
+    try (SegmentedRaftLog raftLog =
+             RetryCacheTestUtil.newSegmentedRaftLog(MEMBER_ID, retryCache, storage, properties)) {
       raftLog.open(RaftLog.INVALID_LOG_INDEX, null);
       checkEntries(raftLog, entries, 0, 650);
       checkEntries(raftLog, newEntries, 100, 100);
-      Assert.assertEquals(newEntries.get(newEntries.size() - 1),
+      Assertions.assertEquals(newEntries.get(newEntries.size() - 1),
           getLastEntry(raftLog));
-      Assert.assertEquals(newEntries.get(newEntries.size() - 1).getIndex(),
+      Assertions.assertEquals(newEntries.get(newEntries.size() - 1).getIndex(),
           raftLog.getFlushIndex());
 
       SegmentedRaftLogCache cache = raftLog.getRaftLogCache();
-      Assert.assertEquals(5, cache.getNumOfSegments());
+      Assertions.assertEquals(5, cache.getNumOfSegments());
     }
   }
 
-  @Test
-  public void testSegmentedRaftLogStateMachineData() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testSegmentedRaftLogStateMachineData(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     final SegmentRange range = new SegmentRange(0, 10, 1, true);
     final List<LogEntryProto> entries = prepareLogEntries(range, null, true, new ArrayList<>());
 
     final SimpleStateMachine4Testing sm = new SimpleStateMachine4Testing();
     try (SegmentedRaftLog raftLog = SegmentedRaftLog.newBuilder()
-        .setMemberId(memberId)
+        .setMemberId(MEMBER_ID)
         .setStateMachine(sm)
         .setStorage(storage)
         .setProperties(properties)
@@ -622,8 +726,11 @@ public class TestSegmentedRaftLog extends BaseTest {
     }
   }
 
-  @Test(expected = TimeoutIOException.class)
-  public void testServerShutdownOnTimeoutIOException() throws Throwable {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testServerShutdownOnTimeoutIOException(Boolean useAsyncFlush, Boolean smSyncFlush) throws Throwable {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     RaftServerConfigKeys.Log.StateMachineData.setSync(properties, true);
     final TimeDuration syncTimeout = TimeDuration.valueOf(100, TimeUnit.MILLISECONDS);
     RaftServerConfigKeys.Log.StateMachineData.setSyncTimeout(properties, syncTimeout);
@@ -645,16 +752,16 @@ public class TestSegmentedRaftLog extends BaseTest {
         LOG.info("Test StateMachine: Ratis log failed notification received as expected.", cause);
 
         LOG.info("Test StateMachine: Transition to PAUSED state.");
-        Assert.assertNotNull(entry);
+        Assertions.assertNotNull(entry);
 
         getLifeCycle().transition(LifeCycle.State.PAUSING);
         getLifeCycle().transition(LifeCycle.State.PAUSED);
       }
     };
 
-    Throwable ex = null; // TimeoutIOException
+    ExecutionException ex;
     try (SegmentedRaftLog raftLog = SegmentedRaftLog.newBuilder()
-        .setMemberId(memberId)
+        .setMemberId(MEMBER_ID)
         .setStateMachine(sm)
         .setStorage(storage)
         .setProperties(properties)
@@ -663,15 +770,10 @@ public class TestSegmentedRaftLog extends BaseTest {
       // SegmentedRaftLogWorker should catch TimeoutIOException
       CompletableFuture<Long> f = raftLog.appendEntry(entry);
       // Wait for async writeStateMachineData to finish
-      try {
-        f.get();
-      } catch (ExecutionException e) {
-        ex = e.getCause();
-      }
+      ex = Assertions.assertThrows(ExecutionException.class, f::get);
     }
-    Assert.assertNotNull(ex);
-    Assert.assertSame(LifeCycle.State.PAUSED, sm.getLifeCycleState());
-    throw ex;
+    Assertions.assertSame(LifeCycle.State.PAUSED, sm.getLifeCycleState());
+    Assertions.assertInstanceOf(TimeoutIOException.class, ex.getCause());
   }
 
   static Thread startAppendEntryThread(RaftLog raftLog, LogEntryProto entry) {
@@ -688,18 +790,22 @@ public class TestSegmentedRaftLog extends BaseTest {
 
   void assertIndices(RaftLog raftLog, long expectedFlushIndex, long expectedNextIndex) {
     LOG.info("assert expectedFlushIndex={}", expectedFlushIndex);
-    Assert.assertEquals(expectedFlushIndex, raftLog.getFlushIndex());
+    Assertions.assertEquals(expectedFlushIndex, raftLog.getFlushIndex());
     LOG.info("assert expectedNextIndex={}", expectedNextIndex);
-    Assert.assertEquals(expectedNextIndex, raftLog.getNextIndex());
+    Assertions.assertEquals(expectedNextIndex, raftLog.getNextIndex());
   }
 
-  void assertIndicesMultipleAttempts(RaftLog raftLog, long expectedFlushIndex, long expectedNextIndex) throws Exception {
+  void assertIndicesMultipleAttempts(RaftLog raftLog, long expectedFlushIndex, long expectedNextIndex)
+      throws Exception {
     JavaUtils.attempt(() -> assertIndices(raftLog, expectedFlushIndex, expectedNextIndex),
         10, HUNDRED_MILLIS, "assertIndices", LOG);
   }
 
-  @Test
-  public void testAsyncFlushPerf1() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testAsyncFlushPerf1(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     List<SegmentRange> ranges = prepareRanges(0, 50, 20000, 0);
     List<LogEntryProto> entries = prepareLogEntries(ranges, null);
 
@@ -721,8 +827,11 @@ public class TestSegmentedRaftLog extends BaseTest {
     }
   }
 
-  @Test
-  public void testAsyncFlushPerf2() throws Exception {
+  @ParameterizedTest
+  @MethodSource("data")
+  public void testAsyncFlushPerf2(Boolean useAsyncFlush, Boolean smSyncFlush) throws Exception {
+    RaftServerConfigKeys.Log.setAsyncFlushEnabled(properties, useAsyncFlush);
+    RaftServerConfigKeys.Log.StateMachineData.setSync(properties, smSyncFlush);
     List<SegmentRange> ranges = prepareRanges(0, 50, 20000, 0);
     List<LogEntryProto> entries = prepareLogEntries(ranges, null);
 
